
div.commentInfo {
  background-image: none;
  font-weight: normal;
  padding: 0.2em;
  background : #f2f2f2;
  border: 1px solid #dfdfdf
}

/* patches */
.diffChunk div.section {
 width: auto;
}

.diffChunk .vcreview_activitysection,
.diffChunk .commentsection {
  margin-left: 3.8em;
}
.diffChunk .vcreview_activitysection {
 margin-top: 1em
}

.diffChunk .commentsection {
 margin-bottom: 1em
}

div.diffComment {
  margin-bottom: 1.8em;
  color: #333;
}

span.diffChunkTitle {
 font-weight: bold;
}

div.diffCmd {
 margin: 1em 0 0;
 padding: 4px;
 border: 1px #AAA solid;
 background-color: #C2C2C2;
 font-weight: bold;
 font-size: 130%;
}

div.diffChunk pre {
 margin: 0.5em 0;
 padding: 0 0.3em;
 border-color: #dfdfdf;
 line-height: 1.2m;
}

div.diffChunk pre span.diffOldLine {
 color:#FF0000;
}

div.diffChunk pre span.diffNewLine {
  color: green;
}

pre.diff {
  font-size: 110%;
}

form.inline, form.inline fieldset {
  display: inline;
}
form.inline button {
  margin: 0;
}

.vcreview_task {
    margin-left: 0.5em;
    padding-left: 0.25em;
    border-left: 2px solid grey;
}
.vcreview_task h4 {
    color: black;
}

.vcreview_task h5 .vcreview_tasktitle {
    font-size: 1.5em;
    /*background-color: #EEEEEE;*/
}

.vcreview_task .vcreview_taskinfo {
    font-size: 0.8em;
    color: grey;
}

.vcreview_task .commentsection h4 {
    font-size: 1.0em;
}

/* the 'add comment' link is in a classless span :( */
.vcreview_task .commentsection > span:last-child {
    padding: 0.15 em;
    font-weight: bolder;
    display: block;
}
.vcreview_task .vcreview_taskstate {
    display: inline-block;
    padding: 0.20em;
    padding-left: 0.4em;
    margin-left: 0.25em;
    padding-right: 0.5em;
    margin-right: 0.5em;
    color: white;
    border-radius: 2px;
    text-shadow: 0 -1px 0 rgba(0,0,0,0.2);
}
.vcreview_task.task_todo .vcreview_taskstate {
    background-color: #AA4444;
}
.vcreview_task.task_done .vcreview_taskstate {
    background-color: #44AA44;
}
.vcreview_task.task_todo {
    background-color: #FFEEEE;
    border-left-color: #AA4444;
}
.vcreview_task.task_done {
    background-color: #EEFFEE;
    border-left-color: #44AA44;
}
.vcreview_task input.task_is_done {
    font-size: 0.8em;
    margin: 0.3em;
    float: right;
    border-radius: 2px;
}
.vcreview_task input.task_is_done:hover {
    cursor: pointer;
}
.vcreview_task h5 {
    font-size: 90%;
}